#include "board.h"

Board::Board( SDL_Surface *screen, int size ) {

	this->setMustPut( true );
	this->setMustRot( false );
	this->setTurn( 1 );
	
	sets = new Set*[ 2 ];
	for ( int i = 0; i < 2; i++ ){
		sets[ i ] = new Set[ 2 ];
		for ( int j = 0; j < 2; j++ ){
			sets[i][j] = Set( size );
			sets[i][j].setWidth( screen->w / 2 - 10 );
			sets[i][j].setHeight( screen->h / 2 - 10);
		}
	}
	sets[0][0].setCor( 5, 5 );
	sets[0][1].setCor( screen->w / 2 + 5, 5 );
	sets[1][0].setCor( 5, screen->h / 2 + 5 );
	sets[1][1].setCor( screen->w / 2 + 5, screen->h / 2 + 5 );
}

Board::~Board() {
	for ( int i = 0; i < 2; i++ )
		delete[] sets[i];
	delete[] sets;
}

void Board::drawTable( SDL_Surface *screen ) {

//	Drawing each set
	for ( int i = 0; i < 2; i++ )
		for ( int j = 0; j < 2; j++ )
			sets[i][j].draw( screen, _c);
	SDL_Flip( screen );
}

int Board::run( SDL_Surface *screen ) {
	this->drawTable( screen );
	return this->startGame( screen );
}

void Board::addedPiece () {
	this->setMustPut( false );
	this->setMustRot( true );
}

void Board::rotated() {
	this->setMustPut( true );
	this->setMustRot( false );
	this->setTurn( this->turn() * -1 ) ;
}

int Board::startGame( SDL_Surface *screen ) {
	
	SDL_Event event;

	while ( 1 ) {
		while ( SDL_PollEvent( &event ) ){
			
			if ( event.type == SDL_QUIT )
				return 0;

			if ( event.type == SDL_KEYDOWN )
				if ( event.key.keysym.sym == SDLK_F4 && ( event.key.keysym.mod != 0 && ( event.key.keysym.mod & KMOD_ALT ) ) )
					return 0;

			if ( event.type == SDL_MOUSEBUTTONDOWN ) {
				for ( int i = 0; i < 2; i++ ){
					for ( int j = 0; j < 2; j++ ){

//						If clicked on a set
						if ( clickOn( event, sets[i][j] ) ){

//							If player must put a piece
							if ( this->mustPut() ){
//								If is was possible to add a piece and the piece added
								if ( sets[i][j].putPiece( event, this->turn() ) ){
//									Tell the program that a piece added
									this->addedPiece();

								}
								
							}
							
//							If the player must rotate a set
							else if ( this->mustRot() ){
//								Rotate the selected set
								sets[i][j].rotate( event );
//								Tell the program that a set is rotated
								this->rotated();
								
//								Drawing the surface, the updating it
								this->drawTable( screen );
								SDL_Flip( screen );
								
//								If a player won the game
								if ( this->checkWinning() ){
//									Set the winner
									this->setWinner( this->checkWinning() );
//									Telling that we have a winner here!
									return 1;
								}
							}

//							Drawing all of the sets
							sets[i][j].draw( screen, _c );
							
//							Updating the surface
							SDL_Flip( screen );

						}
					}
				}
			}

		}
	}
}

void Board::showWinner ( SDL_Surface* screen ) {

//	Initializing ttf library
	TTF_Init();

//	Opening the font
	TTF_Font *font = TTF_OpenFont( "fonts/Arial.ttf", 32 );

//	Color of the text
	SDL_Color color{ 255, 255, 255 };

//	message
	string text;
	text.append( this->winner() ).append( " Won the game!" );

//	Message surface
	SDL_Surface *message = TTF_RenderText_Solid( font, text.c_str(), color );

//	Position of the message
	SDL_Rect pos;
	pos.x = ( screen->w - message->w ) / 2;
	pos.y = ( screen->h - message->h ) / 2;

//	Applying the message to the screen
	SDL_BlitSurface( message, NULL, screen, &pos );

//	Updating the surface
	SDL_Flip( screen );

}

int Board::checkWinning () {
	
//	Combining all sets
	Set temp( 6 );
	for ( int i = 0; i < 2; i++ )
		for ( int j = 0; j < 2; j++ )
			for ( int k = 0; k < 3; k++ )
				for ( int l = 0; l < 3; l++ )
					temp[ i * 3 + k ][ j * 3 + l ] = sets[i][j][k][l];
	
	for ( int i = 0; i < 6; i++ ){

//		Checking for a winner in vertical form
		if ( temp[1][i] == temp[2][i] && temp[2][i] == temp[3][i] && temp[3][i] == temp[4][i] && temp[1][i].color() != 0 )
			if ( temp[0][i] == temp[1][i] || temp[5][i] == temp[1][i] )
				return temp[1][i].color();

//		Checking for a winner in horizontal form
		if ( temp[i][1] == temp[i][2] && temp[i][2] == temp[i][3] && temp[i][3] == temp[i][4] && temp[i][1].color() != 0 )
			if ( temp[i][0] == temp[i][1] || temp[i][5] == temp[i][1] )
				return temp[i][1].color();
	}

//	Checking for a winner in diagonal form
	if ( temp[1][1] == temp[2][2] && temp[2][2] == temp[3][3] && temp[3][3] == temp[4][4] && temp[1][1].color() != 0 )
		if ( temp[0][0] == temp[1][1] || temp[5][5] == temp[1][1] )
			return temp[1][1].color();
	if ( temp[1][4] == temp[2][3] && temp[2][3] == temp[3][2] && temp[3][2] == temp[4][1] && temp[1][4].color() != 0 )
		if ( temp[0][5] == temp[1][4] || temp[5][0] == temp[1][4] )
			return temp[1][4].color();

	return 0;

}

Set *Board::operator []( int pos ) {
	return sets[ pos ];
}

void Board::setMustPut( bool mustPut ) {
	this->_mustPut = mustPut;
}

void Board::setMustRot( bool mustRot ) {
	this->_mustRot = mustRot;
}

void Board::setTurn( int turn ) {
	this->_turn = turn;
}

void Board::setWinner( int winner ) {
	this->_winner = winner;
}

bool Board::mustPut () {
	return this->_mustPut;
}

bool Board::mustRot () {
	return this->_mustRot;
}

int Board::turn () {
	return this->_turn;
}

string Board::winner () {
	if ( this->_winner == 0 )
		return "";
	return ( this->_winner == 1 ? "Red" : "Black" );
}

