#ifndef BOARD_H
#define BOARD_H

#include <string>

#include "SDL/SDL.h"
#include "SDL/SDL_gfxPrimitives.h"
#include "SDL/SDL_ttf.h"

#include "color.h"
#include "set.h"

using namespace std;

class Board{ 
	
	friend bool clickOn( Set );

	private:
		const Color _c = Color( 0, 255, 255, 255);
		
//		Four sets
		Set **sets;

		void drawTable( SDL_Surface* );
		int startGame( SDL_Surface* );

//		for player state
		bool _mustPut, _mustRot;
	
//		Represents turn
		int _turn;

//		Represents winner
		int _winner;

		void setMustPut( bool );
		void setMustRot( bool );
		void setTurn( int );
		void setWinner( int );

		int checkWinning();

//		These functions change the board state
		void addedPiece();
		void rotated();

	public:

//		Constructor and destructor
		Board( SDL_Surface*, int );
		~Board();

//		Runs the game
		int run( SDL_Surface* );

//		Show the winner on screen
		void showWinner( SDL_Surface* );

		Set *operator[]( int );

		bool mustPut();
		bool mustRot();
		int turn();
		string winner();

};


/*
This Function checks if the click was inside the set
*/

inline bool clickOn( SDL_Event event, const Set &set ) {

	if ( event.button.x >= set.x() && event.button.x <= set.x() + set.width() && event.button.y >= set.y() && event.button.y <= set.y() + set.height() ){
		return true;
	}

	return false;

}

#endif

