#include "set.h"

Set::Set( int size ) {
	this->setSize( size );
	set = new Block*[ size ];
	for ( int i = 0; i < size; i++ ){
		set[i] = new Block[ size ];
		for ( int j = 0; j < size; j++ )
			set[i][j].setColor( 0 );
	}
}

Set::Set(const Set& rhs) {
	this->setCor(rhs.x() , rhs.y());
	this->setWidth(rhs.width());
	this->setHeight(rhs.height());
	this->setSize( rhs.size() );
	set = new Block*[ this->size() ];
	for ( int i = 0; i < this->size(); i++ ){
		set[i] = new Block[ this->size() ];
		for ( int j = 0; j < this->size(); j++ )
			set[i][j].setColor( rhs[i][j].color() );
	}
}

void Set::drawCircle( SDL_Surface* screen, int x, int y ) {

//	If intended block was red
	if ( set[y][x].color() == 1 )
		filledEllipseRGBA( screen, x * this->width() / 3 + this->width() / 6 + this->x(), y * this->height() / 3 + this->height() / 6 + this->y(), this->width() / 6 - 5, this->height() / 6 - 5, 255, 0, 0, 255 );

//	If intended block was black
	else if ( set[y][x].color() == -1 )
		filledEllipseRGBA( screen, x * this->width() / 3 + this->width() / 6 + this->x(), y * this->height() / 3 + this->height() / 6 + this->y(), this->width() / 6 - 5, this->height() / 6 - 5, 0, 0, 0, 255 );

}

void Set::draw( SDL_Surface *screen, const Color& color ) {

//	Drawing the set
	boxRGBA( screen, this->x(), this->y(), this->x() + this->width(), this->y() + this->height(), color.red(), color.green(), color.blue(), color.alpha() );

//	Drawing the lines
	for ( int i = 1; i < 3; i++ ) {
			lineRGBA( screen, this->x(), this->height() / 3 * i + this->y(), this->x() + this->width(), this->height() / 3 * i + this->y(), 255 - color.red(), 255 - color.green(), 255 - color.blue(), color.alpha() );
			lineRGBA( screen, this->width() / 3 * i + this->x(), this->y(), this->width() / 3 * i + this->x(), this->y() + this->height(), 255 - color.red(), 255 - color.green(), 255 - color.blue(), color.alpha() );
	}

//	Drawing all of the blocks
	for ( int i = 0; i < 3; i++ ){
		for ( int j = 0; j < 3; j++ ){
			drawCircle( screen, j, i );
		}
	}

}

bool Set::putPiece( SDL_Event event, int turn ) {
	int x = ( event.button.x - this->x() ) / ( this->width() / 3 );
	int y = ( event.button.y - this->y() ) / ( this->height() / 3 );
	if ( set[y][x].color() != 0 )
		return false;
	set[y][x].setColor( turn );
	return true;
}

void Set::rotate( SDL_Event event ) {

	Block **temp;
	temp = new Block*[ this->size() ];
	for ( int i = 0; i < this->size(); i++ ){
		temp[i] = new Block[ this->size() ];
	}

//	If user wants to rotate the set clockwise
	if ( event.button.button == SDL_BUTTON_RIGHT )
		for ( int i = 0; i < this->size(); i++ )
			for ( int j = 0; j < this->size(); j++ )
				temp[ i ][ j ].setColor( set[ this->size() - j - 1 ][ i ].color() );

//	If user wants to rotate the set counter-clockwise
	if ( event.button.button == SDL_BUTTON_LEFT )
		for ( int i = 0; i < this->size(); i++ )
			for ( int j = 0; j < this->size(); j++ )
				temp[i][j].setColor( set[ j ][ this->size() - i - 1 ].color() );
	
	for ( int i = 0; i < this->size(); i++ )
		for ( int j = 0; j < this->size(); j++ )
			set[i][j].setColor( temp[i][j].color() );
	
	for ( int i = 0; i < this->size(); i++ )
		delete[] temp[i];
	delete[] temp;
}

Block *Set::operator[]( int pos ) const{
	return set[ pos ];
}

Set& Set::operator=(const Set& rhs) {
	this->setCor(rhs.x() , rhs.y());
	this->setWidth(rhs.width());
	this->setHeight(rhs.height());
	if (rhs.size() !=  this->size())
	{
		for ( int i = 0; i < this->size(); i++ )
			delete[] set[i];
		delete[] set;	
		this->setSize(rhs.size());
		set = new Block*[ this->size() ];
		for ( int i = 0; i < this->size(); i++ ){
			set[i] = new Block[ this->size() ];
			for ( int j = 0; j < this->size(); j++ )
				set[i][j].setColor( rhs[i][j].color() );
		}
	}
	else
	{
		for ( int i = 0; i < this->size(); i++ )
			for ( int j = 0; j < this->size(); j++ )
				set[i][j].setColor( rhs[i][j].color() );
	}
	return *this;
}

void Set::setX( int x ) {
	this->_x = x;
}

void Set::setY( int y ) {
	this->_y = y;
}

void Set::setCor( int x, int y ) {
	this->setX( x );
	this->setY( y );
}

void Set::setWidth( int width ) {
	this->_width = width;
}

void Set::setHeight( int height ) {
	this->_height = height;
}

void Set::setSize( int size ) {
	this->_size = size;
}

int Set::x() const{
	return this->_x;
}

int Set::y() const{
	return this->_y;
}

int Set::width() const{
	return this->_width;
}

int Set::height() const{
	return this->_height;
}

int Set::size() const{
	return this->_size;
}

Set::~Set()
{
	for ( int i = 0; i < this->size(); i++ )
			delete[] set[i];
	delete[] set;
}
